package leigh.stack.zero;

import leigh.mecha.log.MechaLogger;
import leigh.mecha.log.MechaLoggerFactory;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.HashSet;

/**
 * Implementation of a Zero client.
 *
 * @author Alex Leigh
 */
public class ZeroClient implements Runnable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(ZeroClient.class);
    public static int ZERO_PORT = 5200;
    private String url = null;
    private final HashSet<Personality> capabilities = new HashSet<>();

    @Override
    public void run() {
        try {
            DatagramSocket serverSocket = new DatagramSocket(ZERO_PORT);
            serverSocket.setBroadcast(true);
            serverSocket.setReuseAddress(true);
            byte[] receiveData = new byte[1500];
            //noinspection InfiniteLoopStatement

            while (true) {
                DatagramPacket receivePacket = new DatagramPacket(receiveData, receiveData.length);
                serverSocket.receive(receivePacket);
                //  HexDump.hexDump(receivePacket.getData(), 0, receivePacket.getLength());
                // FIXME: If this throws we'll break the loop
                Beacon b = new Beacon(receivePacket.getData());
                processBeacon(b);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void processBeacon(Beacon b) {
        logger.info("Received beacon: {}", b);
        if (this.url == null) {
            reconfigure(b.getZaiUrl());
            // TODO: How about some stricmp
        } else if (this.url.equals(b.getZaiUrl()) && b.isAuthoritative()) {
            reconfigure(b.getZaiUrl());
        }
    }

    /**
     * Initiate a reconfiguration to a new activation index URL.
     *
     * @param zaiUrl The activation index URL.
     */
    private void reconfigure(String zaiUrl) {
        this.url = zaiUrl;
        logger.info("Reconfiguration event. [ZAI: {}]", this.url);

        HashSet<String> capNames = new HashSet<>();
        for (Personality p : capabilities) {
            capNames.add(p.getName());
        }

        logger.info("Capabilities: {}", capNames);
    }

    public HashSet<Personality> getCapabilities() {
        return capabilities;
    }
}
